@@ -2,25 +2,30 @@
 from __future__ import absolute_import
 
 import inspect
+from threading import RLock
 
 from thriftworker.utils.decorators import cached_property
+from thriftworker.utils.imports import instantiate
 from thriftworker.app import ThriftWorker
 
+from thriftpool.processes.manager import Manager
 from thriftpool.app.config import Configuration
 from thriftpool.exceptions import RegistrationError
 from thriftpool.utils.mixin import SubclassMixin
-from thriftpool.utils.imports import symbol_by_name
 
 from ._state import set_current_app
 
-try:
-    from billiard.util import register_after_fork
-except ImportError:
-    register_after_fork = lambda *args: None
-
 __all__ = ['ThriftPool']
 
 
+def _unpickle_app(cls, changes, slots):
+    app = cls()
+    app.config.update(changes)
+    app.slots = slots
+    app.loader.after_unpickling()
+    return app
+
+
 class ThriftPool(SubclassMixin):
     """Main entry point for this application."""
 
@@ -28,17 +33,38 @@ class ThriftPool(SubclassMixin):
     #: for it.
     loader_cls = 'thriftpool.app.loader:Loader'
 
+    #: What class should be used for logging setup.
+    logging_cls = 'thriftpool.app.log:Logging'
+
+    #: Repository for registered slots.
+    repo_cls = 'thriftpool.app.slots:Repository'
+
+    #: Manager controller class.
+    manager_cls = 'thriftpool.controllers.manager:ManagerController'
+
+    #: Worker controller class.
+    worker_cls = 'thriftpool.controllers.worker:WorkerController'
+
+    #: Specify daemonizing behavior.
+    daemon_cls = 'thriftpool.app.daemon:Daemon'
+
+    #: Store active requests here.
+    request_stack_cls = 'thriftpool.request.stack:RequestStack'
+
     def __init__(self):
-        # we must delete some entries after work
-        register_after_fork(self, self._after_fork)
+        self._finalized = False
+        self._finalize_mutex = RLock()
         super(ThriftPool, self).__init__()
 
         # set current application as default
         set_current_app(self)
 
-    def _after_fork(self, obj_):
-        # Reset needed resources after fork."
-        pass
+    def __reduce__(self):
+        # Reduce only pickles the configuration changes,
+        # so the default configuration doesn't have to be passed
+        # between processes.
+        return (_unpickle_app, (self.__class__, self.config._changes,
+                                self.slots))
 
     @cached_property
     def Loader(self):
@@ -61,7 +87,7 @@ class ThriftPool(SubclassMixin):
         logging.
 
         """
-        return self.subclass_with_self('thriftpool.app.log:Logging')
+        return self.subclass_with_self(self.logging_cls)
 
     @cached_property
     def log(self):
@@ -71,7 +97,7 @@ class ThriftPool(SubclassMixin):
     @cached_property
     def Repository(self):
         """Create bounded slots repository from :class:`.slots:Repository`."""
-        return self.subclass_with_self('thriftpool.app.slots:Repository')
+        return self.subclass_with_self(self.repo_cls)
 
     @cached_property
     def slots(self):
@@ -80,32 +106,53 @@ class ThriftPool(SubclassMixin):
 
     def finalize(self):
         """Make some steps before application startup."""
-        # Setup logging for whole application.
-        self.log.setup()
-        # Load all needed modules.
-        self.loader.preload_modules()
-        # Register existed services.
-        for params in self.config.SLOTS:
-            self.slots.register(**params)
+        with self._finalize_mutex:
+            if self._finalized:
+                return
+            # Setup logging for whole application.
+            self.log.setup()
+            # Load all needed modules.
+            self.loader.preload_modules()
+            # Register existed services.
+            for params in self.config.SLOTS:
+                self.slots.register(**params)
+            self._finalized = True
+
+    @property
+    def loop(self):
+        return self.thriftworker.loop
 
     @cached_property
     def protocol_factory(self):
         """Create handler instance."""
-        ProtocolFactory = symbol_by_name(self.config.PROTOCOL_FACTORY_CLS)
-        return ProtocolFactory()
+        return instantiate(self.config.PROTOCOL_FACTORY_CLS)
 
     @cached_property
     def thriftworker(self):
         return ThriftWorker(port_range=self.config.SERVICE_PORT_RANGE,
-                            protocol_factory=self.protocol_factory)
+                            protocol_factory=self.protocol_factory,
+                            pool_size=self.config.CONCURRENCY)
+
+    @cached_property
+    def gaffer_manager(self):
+        """Create process manager."""
+        return Manager(loop=self.loop)
+
+    @property
+    def env(self):
+        return self.thriftworker.env
+
+    @cached_property
+    def ManagerController(self):
+        return self.subclass_with_self(self.manager_cls)
 
     @cached_property
     def WorkerController(self):
-        return self.subclass_with_self('thriftpool.controllers.worker:WorkerController')
+        return self.subclass_with_self(self.worker_cls)
 
     @cached_property
-    def Worker(self):
-        return self.subclass_with_self('thriftpool.app.worker:Worker')
+    def Daemon(self):
+        return self.subclass_with_self(self.daemon_cls)
 
     def register(self, *args, **options):
         """Register new handler."""
@@ -139,5 +186,4 @@ class ThriftPool(SubclassMixin):
     @cached_property
     def request_stack(self):
         """Store current requests."""
-        RequestStack = symbol_by_name('thriftpool.request.stack:RequestStack')
-        return RequestStack()
+        return instantiate(self.request_stack_cls)
